import { Component, OnInit, AfterViewInit, ViewChild } from '@angular/core';
import { AfterContentInit } from '@angular/core/src/metadata/lifecycle_hooks';
import { Router } from '@angular/router';
import { Sensor } from '../../model';

import { Store, select } from '@ngrx/store';

import * as SensorStore from '../../store';

import { RPCClientError } from '@loafer/ng-rpc/protocol';
import * as ListStore from '../../store/list';
import { sensorListSelector } from '../../store';
import { Domain } from 'packages/domain/model';
import { AuthSelector } from 'packages/member/store';
import { Page, PageParams } from 'app/commons/model';
import { Target } from 'packages/target/model';
import { SettingComponent } from '../setting/setting.component';
import { SelectItem } from 'primeng/primeng';

@Component({
  selector: 'of-sensor-list',
  templateUrl: './list.component.html',
  styleUrls: ['./list.component.scss']
})
export class ListComponent implements OnInit, AfterContentInit {

  sensorList$ = this.store.pipe(select(sensorListSelector.select('page')));
  PAGE_SIZE = '99999';
  totalLength = 0;
  sensorSettingDisplay = false;

  sensors: Sensor[];
  target: Target = null;

  targetSensor;

  // filter
  targetOptions: SelectItem[];
  selectedTargets: string[] = [];

  constructor(private router: Router,
    private store: Store<ListStore.State>,
  ) { }

  ngOnInit() {
    this.sensorList$.subscribe(
      (page: Page) => {
        if (page != null) {
          this.sensors = page.content;
          this.generateSensorMap();
        }
      },
      (error: RPCClientError) => {
        console.log(error.response.message);
      }
    );
  }

  ngAfterContentInit() {
    this.getSensors(0);
  }

  getSensors(pageIndex: number) {
    this.store.select(AuthSelector.select('domain')).subscribe(
      (domain: Domain) => {
        const pageParams: PageParams = {
          pageNo: pageIndex + '',
          countPerPage: this.PAGE_SIZE,
          sortCol: 'id',
          sortDirection: 'descending'
        };

        this.store.dispatch(new ListStore.ReadAllByDomain({ domain, pageParams }));
      },
      (error) => {
        console.log(error);
      }
    );
  }

  generateSensorMap() {
    const tempList = [];
    for (const sensor of this.sensors) {
      const targetNode = this.existTarget(sensor, tempList);
      if (targetNode === null) {
        const sensorsNode = [];
        sensorsNode.push(sensor);
        const node = {
          target: sensor.target,
          sensors: sensorsNode,
        };

        tempList.push(node);
      } else {
        targetNode.sensors.push(sensor);
      }
    }
    this.totalLength = tempList.length;
    this.targetSensor = tempList;

    this.generateTargetFilter();
  }

  generateTargetFilter() {
    if (this.targetOptions) {
      return;
    }
    const tempList = [];
    for (const data of this.targetSensor) {
      tempList.push({ label: data.target.displayName, value: data.target.id });
    }
    this.targetOptions = tempList;
  }

  existTarget(sensor: Sensor, tempList) {
    let targetNode = null;
    for (const node of tempList) {
      if (node.target.id === sensor.target.id) {
        targetNode = node;
      }
    }
    return targetNode;
  }

  onAddSensor() {
    this.target = null;
    this.sensorSettingDisplay = true;
  }

  onSensorSettingClose() {
    this.sensorSettingDisplay = false;
  }
  onSensorClick(sensor) {
    this.router.navigate(['sensor', sensor.id, 'info']);
  }

  onSearch() {

  }




  // Test
  onAddSensorWithTarget() {
    this.target = {
      id: 1,
      createDate: new Date(),
      displayName: 'Alredy selected target111',
      description: 'Desc..',
    };
    this.sensorSettingDisplay = true;
  }

  onAddSensorWithTarget2() {
    this.target = {
      id: 2,
      createDate: new Date(),
      displayName: 'Alredy selected target222',
      description: 'Desc..',
    };
    this.sensorSettingDisplay = true;
  }
}

